package com.smardec.demo;

import com.smardec.demo.regviewer.RegViewer;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class MainFrame extends JFrame{
    private JTextField messageCaptionField;
    private JTextArea messageText;
    private JComboBox messageIconBox;
    private JComboBox messageStyleBox;
    private JComboBox toneBox;

    private RegViewer regViewer;
    private boolean inProgress = false;

    public MainFrame() throws HeadlessException{
        super("J2Native Demo");
        setIconImage(new ImageIcon(MainFrame.class.getResource("logo.png")).getImage());
        Container contentPane = getContentPane();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int frameWidth = 600;
        int frameHeight = 400;
        setBounds((screenSize.width - frameWidth) / 2, (screenSize.height - frameHeight) / 2, frameWidth, frameHeight);

        final JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);

        JPanel firstTab = createFirstTab();
        JPanel thirdTab = createThirdTab();

        tabbedPane.addTab("Simple", firstTab);
        tabbedPane.addTab("Registry Viewer", thirdTab);

        tabbedPane.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e){
                if(tabbedPane.getSelectedIndex() == 1){
                    if(!regViewer.initFunctions()){
                        return;
                    }
                    if(inProgress){
                        return;
                    }
                    inProgress = true;
                    new Thread(new Runnable(){
                        public void run(){
                            Cursor oldCursor = getCursor();
                            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                            regViewer.loadRegistry();
                            setCursor(oldCursor);
                            inProgress = false;
                        }
                    }).start();
                }
            }
        });

        contentPane.add(tabbedPane, BorderLayout.CENTER);
    }

    private JPanel createFirstTab(){
        JPanel tab = new JPanel();
        tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));

        tab.add(Box.createVerticalStrut(10));

        JLabel tabNameLabel = new JLabel("Simple library method invocation demonstation.", JLabel.CENTER);
        tabNameLabel.setForeground(Color.black);
        JPanel jPanel = new JPanel(new FlowLayout());
        jPanel.add(tabNameLabel);
        tab.add(jPanel);

        tab.add(Box.createVerticalStrut(10));

        JPanel messageBeepPanel = new JPanel(new GridBagLayout());
        TitledBorder titledBorder = BorderFactory.createTitledBorder("I. User32.dll :: MessageBeep()");
        titledBorder.setTitleColor(Color.black);
        messageBeepPanel.setBorder(titledBorder);

        JLabel toneLabel = new JLabel("Tone:");
        toneLabel.setForeground(Color.black);
        constrain(messageBeepPanel, toneLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        toneBox = new JComboBox(new Object[]{"MB_OK", "MB_ICONHAND", "MB_ICONQUESTION", "MB_ICONEXCLAMATION", "MB_ICONASTERISK"});
        constrain(messageBeepPanel, toneBox, 1, 0, 2, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        JButton beepButton = new JButton("Invoke");
        beepButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                J2NMain.getInstance().messageBeep(toneBox.getSelectedIndex() << 4);
            }
        });
        constrain(messageBeepPanel, beepButton, 2, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        tab.add(messageBeepPanel);

        tab.add(Box.createVerticalStrut(10));

        JPanel messageBoxPanel = new JPanel(new GridBagLayout());
        titledBorder = BorderFactory.createTitledBorder("II. User32.dll :: MessageBox()");
        titledBorder.setTitleColor(Color.black);
        messageBoxPanel.setBorder(titledBorder);

        JLabel captionLabel = new JLabel("Caption:");
        captionLabel.setForeground(Color.black);
        constrain(messageBoxPanel, captionLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        messageCaptionField = new JTextField();
        constrain(messageBoxPanel, messageCaptionField, 1, 0, 3, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        JLabel textLabel = new JLabel("Text:");
        textLabel.setForeground(Color.black);
        constrain(messageBoxPanel, textLabel, 0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        messageText = new JTextArea();
        JScrollPane textScroll = new JScrollPane(messageText);
        constrain(messageBoxPanel, textScroll, 1, 1, 3, 3, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1.0, 1.0, 5, 5, 5, 5);

        JLabel iconLabel = new JLabel("Icon:");
        iconLabel.setForeground(Color.black);
        constrain(messageBoxPanel, iconLabel, 0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        messageIconBox = new JComboBox(new Object[]{"MB_OK", "MB_ICONHAND", "MB_ICONQUESTION", "MB_ICONEXCLAMATION", "MB_ICONASTERISK"});
        constrain(messageBoxPanel, messageIconBox, 1, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 0.5, 0.0, 5, 5, 5, 5);

        JLabel styleLabel = new JLabel("Style:");
        styleLabel.setForeground(Color.black);
        constrain(messageBoxPanel, styleLabel, 2, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 10, 5, 5);

        messageStyleBox = new JComboBox(new Object[]{"MB_OK", "MB_OKCANCEL", "MB_ABORTRETRYIGNORE", "MB_YESNOCANCEL",
                                                     "MB_YESNO", "MB_RETRYCANCEL"});
        constrain(messageBoxPanel, messageStyleBox, 3, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 0.5, 0.0, 5, 5, 5, 5);

        JButton messageButton = new JButton("Invoke");
        messageButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                J2NMain.getInstance().messageBox(messageCaptionField.getText(),
                                                 messageText.getText(),
                                                 messageStyleBox.getSelectedIndex() | (messageIconBox.getSelectedIndex() << 4));
            }
        });
        constrain(messageBoxPanel, messageButton, 3, 5, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        tab.add(messageBoxPanel);

        return tab;
    }

    private JPanel createThirdTab(){
        JPanel tab = new JPanel(new GridBagLayout());

        JLabel tabNameLabel = new JLabel("Working with a Windows registry using J2Native.", JLabel.CENTER);
        tabNameLabel.setForeground(Color.black);
        constrain(tab, tabNameLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0.0, 0.0, 5, 5, 5, 5);

        regViewer = new RegViewer();
        constrain(tab, regViewer, 0, 1, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);


        return tab;
    }

    private static void constrain(Container container, Component component, int x, int y, int width, int height, int fill, int anchor, double weightx, double weighty, int insetTop, int insetLeft, int insetBottom, int insetRight){
        Insets insets = new Insets(insetTop, insetLeft, insetBottom, insetRight);
        GridBagConstraints gbc = new GridBagConstraints(x, y, width, height, weightx, weighty, anchor, fill, insets, 0, 0);
        LayoutManager lm = container.getLayout();
        GridBagLayout gbl = (GridBagLayout)lm;
        gbl.setConstraints(component, gbc);
        container.add(component);
    }
}
